@@ -1,5 +1,5 @@
 #!/bin/bash
-# $Id: 986b3b4760d83d8029ab887f50e5ff184bd07086 $
+# $Id: 2cfa0fdbb7dbd9a8ec1d91e6b7991c310ae3afa8 $
 
 COPY_BINARIES=false
 CPIO_ARGS="--quiet --null -o -H newc --owner root:root --force-local"
@@ -483,6 +483,7 @@ append_base_layout() {
 	isTrue "${UNIONFS}" && build_parameters+=( --unionfs ) || build_parameters+=( --no-unionfs )
 	isTrue "${ZFS}" && build_parameters+=( --zfs ) || build_parameters+=( --no-zfs )
 	isTrue "${SPLASH}" && build_parameters+=( --splash ) || build_parameters+=( --no-splash )
+	isTrue "${PLYMOUTH}" && build_parameters+=( --plymouth ) || build_parameters+=( --no-plymouth )
 	isTrue "${STRACE}" && build_parameters+=( --strace ) || build_parameters+=( --no-strace )
 	isTrue "${KEYCTL}" && build_parameters+=( --keyctl ) || build_parameters+=( --no-keyctl )
 	isTrue "${GPG}" && build_parameters+=( --gpg ) || build_parameters+=( --no-gpg )
@@ -1330,6 +1331,61 @@ append_splash() {
 	fi
 }
 
+append_plymouth() {
+	local PN=plymouth
+	local TDIR="${TEMP}/initramfs-${PN}-temp"
+	if [ -d "${TDIR}" ]
+	then
+		rm -r "${TDIR}" || gen_die "Failed to clean out existing '${TDIR}'!"
+	fi
+
+	mkdir "${TDIR}" || gen_die "Failed to create '${TDIR}'!"
+	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
+
+	# set plymouth theme
+	if [ -n "${PLYMOUTH_THEME}" ]
+	then
+		plymouth-set-default-theme ${PLYMOUTH_THEME} || gen_die "Failed to set default plymouth theme!"
+	fi
+	if [ -z "${PLYMOUTH_THEME}" -a -e /etc/plymouth/plymouthd.conf ]
+	then
+		PLYMOUTH_THEME=$(plymouth-set-default-theme) || gen_die "Failed to set default plymouth theme!"
+	fi
+	if [ -z "${PLYMOUTH_THEME}" ]
+	then
+		PLYMOUTH_THEME=text
+	fi
+
+	print_info 1 "$(get_indent 1)>> Installing plymouth [ using the '${PLYMOUTH_THEME}' theme ]..."
+
+	/usr/libexec/plymouth/plymouth-populate-initrd -t "${TDIR}" \
+		|| gen_die "Failed to build plymouth cpio archive!"
+
+	# can probably get rid of this; depends if plymouth was built with static libs
+	# rm -f "${TDIR}"/lib*/{ld*,libc*,libz*} \
+		# || gen_die "Failed to clean up plymouth cpio archive!"
+
+	ln -sf "${PLYMOUTH_THEME}/${PLYMOUTH_THEME}.plymouth" "${TDIR}/usr/share/plymouth/themes/default.plymouth" \
+		|| gen_die "Failed to set the default plymouth theme!"
+
+	# include required udev rules
+	mkdir -p "${TDIR}"/usr/lib/udev/rules.d || gen_die "Failed to create '${TDIR}/usr/lib/udev/rules.d'!"
+	cp -aL /lib/udev/rules.d/70-uaccess.rules "${TDIR}/usr/lib/udev/rules.d" || gen_die "Failed to copy '70-uaccess.rules'!"
+	cp -aL /lib/udev/rules.d/71-seat.rules "${TDIR}/usr/lib/udev/rules.d" || gen_die "Failed to copy '71-seat.rules'!"
+
+	# clean up
+	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
+	log_future_cpio_content
+	find . -print0 | "${CPIO_COMMAND}" ${CPIO_ARGS} --append -F "${CPIO_ARCHIVE}" \
+		|| gen_die "Failed to append ${PN} to cpio!"
+
+	cd "${TEMP}" || die "Failed to chdir to '${TEMP}'!"
+	if isTrue "${CLEANUP}"
+	then
+		rm -rf "${TDIR}"
+	fi
+}
+
 append_strace() {
 	local PN=strace
 	local TDIR="${TEMP}/initramfs-${PN}-temp"
@@ -1706,15 +1762,42 @@ append_firmware() {
 
 	mkdir -p "${TDIR}"/lib/firmware || gen_die "Failed to create '${TDIR}/lib/firmware'!"
 
-	if [ ${#FIRMWARE_FILES[@]} -gt 0 ]
+	local -a fwlist=()
+
+	if isTrue "${ALLFIRMWARE}"
 	then
-		pushd "${FIRMWARE_DIR}" &>/dev/null || gen_die "Failed to chdir to '${FIRMWARE_DIR}'!"
-		cp -rL --parents --target-directory="${TDIR}/lib/firmware" "${FIRMWARE_FILES[@]}" 2>/dev/null \
-			|| gen_die "Failed to copy firmware files (${FIRMWARE_FILES}) to '${TDIR}/lib/firmware'!"
-		popd &>/dev/null || gen_die "Failed to chdir!"
-	else
 		cp -a "${FIRMWARE_DIR}"/* "${TDIR}"/lib/firmware/ 2>/dev/null \
 			|| gen_die "Failed to copy firmware files to '${TDIR}/lib/firmware'!"
+	elif [ ${#FIRMWARE_FILES[@]} -gt 0 ]
+	then
+		fwlist=( "${FIRMWARE_FILES[@]}" )
+	else
+		local myfw=
+		while IFS= read -r -u 3 myfw
+		do
+			if [ -z "${myfw}" ]
+			then
+				gen_die "modinfo error!"
+			fi
+
+			if [ ! -f "${FIRMWARE_DIR}/${myfw}" ]
+			then
+				print_warning 3 "$(get_indent 3) - ${myfw} is missing; Ignoring ..."
+				continue
+			fi
+
+			fwlist+=( "${myfw}" )
+		done 3< <( (
+			modinfo -b "${KERNEL_MODULES_PREFIX%/}" -k "${KV}" -F firmware $(mod_dep_list) 2>/dev/null || echo
+		) | sort | uniq )
+	fi
+
+	if [ ${#fwlist[@]} -gt 0 ]
+	then
+		pushd "${FIRMWARE_DIR}" &>/dev/null || gen_die "Failed to chdir to '${FIRMWARE_DIR}'!"
+		cp -rL --parents --target-directory="${TDIR}/lib/firmware" "${fwlist[@]}" 2>/dev/null \
+			|| gen_die "Failed to copy firmware files to '${TDIR}/lib/firmware'!"
+		popd &>/dev/null || gen_die "Failed to chdir!"
 	fi
 
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
@@ -1808,34 +1891,42 @@ append_modules() {
 		gen_die "${error_message}"
 	fi
 
-	determine_KEXT
-
 	cd "${modules_srcdir}" || gen_die "Failed to chdir to '${modules_srcdir}'!"
 
-	print_info 2 "$(get_indent 2)modules: >> Copying modules from '${modules_srcdir}' to initramfs ..."
+	print_info 2 "$(get_indent 2)modules: >> Searching modules in '${modules_srcdir}' ..."
 
-	local i= mymod=
-	local n_copied_modules=0
-	for i in $(gen_dep_list)
+	local mymod=
+	local -a modlist=()
+	while IFS= read -r -u 3 mymod
 	do
-		mymod=$(find . -name "${i}${KEXT}" 2>/dev/null | head -n 1)
 		if [ -z "${mymod}" ]
 		then
-			print_warning 3 "$(get_indent 3) - ${i}${KEXT} not found; Skipping ..."
-			continue;
+			gen_die "modinfo error!"
 		fi
 
-		print_info 3 "$(get_indent 3) - Copying ${i}${KEXT} ..."
-		cp -ax --parents --target-directory "${modules_dstdir}" "${mymod}" 2>/dev/null \
-			|| gen_die "Failed to copy '${modules_srcdir}/${mymod}' to '${modules_dstdir}'!"
-		n_copied_modules=$[$n_copied_modules+1]
-	done
+		if [ "${mymod}" = '(builtin)' ]
+		then
+			continue
+		fi
 
-	if [ ${n_copied_modules} -eq 0 ]
+		if [ ! -f "${mymod}" ]
+		then
+			gen_die "Module '${i}${KEXT}' is missing!"
+		fi
+
+		modlist+=( "${mymod/#${modules_srcdir}\//}" )
+	done 3< <( (
+		modinfo -b "${KERNEL_MODULES_PREFIX%/}" -k "${KV}" -F filename $(mod_dep_list) 2>/dev/null || echo
+	) | sort | uniq )
+
+	if [ ${#modlist[@]} -gt 0 ]
 	then
-		print_warning 1 "$(get_indent 2)modules: ${n_copied_modules} modules copied. Is that correct?"
+		print_info 2 "$(get_indent 2)modules: >> Copying modules from '${modules_srcdir}' to initramfs ..."
+		cp -ax --parents --target-directory "${modules_dstdir}" "${modlist[@]}" 2>/dev/null \
+			|| gen_die "Failed to copy modules!"
+		print_info 2 "$(get_indent 2)modules: ${#modlist[@]} modules copied!"
 	else
-		print_info 2 "$(get_indent 2)modules: ${n_copied_modules} modules copied!"
+		print_warning 1 "$(get_indent 2)modules: 0 modules copied. Is that correct?"
 	fi
 
 	cp -ax --parents --target-directory "${modules_dstdir}" modules* 2>/dev/null \
@@ -2052,16 +2143,6 @@ append_data() {
 create_initramfs() {
 	print_info 1 "initramfs: >> Initializing ..."
 
-	if ! isTrue "${BUILD_KERNEL}"
-	then
-		# Check early for suitable kmod
-		determine_KEXT
-		if ! isTrue "$(is_kext_supported_by_kmod "${KEXT}")"
-		then
-			gen_die "${KMOD_CMD} does not support chosen module compression algorithm. Please re-emerge sys-apps/kmod with USE=$(get_kext_kmod_use_flag "${KEXT}") enabled or adjust CONFIG_MODULE_COMPRESS_* kernel option!"
-		fi
-	fi
-
 	# Create empty cpio
 	CPIO_ARCHIVE="${TMPDIR}/${GK_FILENAME_TEMP_INITRAMFS}"
 	append_data 'devices' # WARNING, must be first!
@@ -2086,6 +2167,7 @@ create_initramfs() {
 	append_data 'modprobed'
 	append_data 'multipath' "${MULTIPATH}"
 	append_data 'splash' "${SPLASH}"
+	append_data 'plymouth' "${PLYMOUTH}"
 	append_data 'strace' "${STRACE}"
 	append_data 'unionfs_fuse' "${UNIONFS}"
 	append_data 'xfsprogs' "${XFSPROGS}"
